"""
Adonis API client implementation.
"""

from urllib.request import urlopen, Request, urljoin
import logging
import json
import datetime
from typing import Dict, Any, Optional

from .exceptions import AdonisAuthenticationDeniedError, AdonisError
from .crew import Crew
from .const import USER_AGENT, REQUIREMENTS_VIEW, CERTIFICATES_VIEW
from .cache import ResponseCache

from dateutil.relativedelta import relativedelta

logger = logging.getLogger(__name__)

CREW_PORTAL = 0
INTEGRATION = 1


class Adonis:
    """
    Client for interacting with the Adonis API.

    This class provides methods for fetching crew data, managing certificates,
    and working with competencies.
    """

    def __init__(
        self,
        crew_portal_base_url: str,
        integration_base_url: str,
        login: str,
        password: str,
        cache_enabled: bool = True,
        cache_dir: Optional[str] = None,
        cache_expiry: int = 3600,
    ):
        self.crew_portal_base_url = crew_portal_base_url
        self.integration_base_url = integration_base_url
        self.login = login
        self.password = password

        # Initialize cache if enabled
        self.cache_enabled = cache_enabled
        if cache_enabled:
            self.cache = ResponseCache(cache_dir=cache_dir, expiry_seconds=cache_expiry)
        else:
            self.cache = None

    @classmethod
    def fromConfig(cls, config):
        """
        Create an Adonis client from a Config object.

        Args:
            config: Configuration object

        Returns:
            An initialized Adonis client
        """
        return cls(
            config.crew_portal_base_url,
            config.integration_base_url,
            config.login,
            config.password,
            cache_enabled=config.cache_enabled,
            cache_dir=config.cache_dir,
            cache_expiry=config.cache_expiry,
        )

    def request(
        self,
        endpoint: str,
        payload: Dict[str, Any],
        add_token: bool = True,
        wrap_request: bool = True,
        extract_response: bool = True,
        api: int = CREW_PORTAL,
        print_payload: bool = False,
        use_cache: bool = False,
    ) -> Dict[str, Any]:
        """
        Make a request to the Adonis API.

        Args:
            endpoint: API endpoint
            payload: Request parameters
            add_token: Whether to add authentication token
            wrap_request: Whether to wrap payload in a "request" object
            extract_response: Whether to extract the response data
            api: Which API to use (CREW_PORTAL or INTEGRATION)
            print_payload: Whether to print the request payload (for debugging)
            use_cache: Whether to use cache for this request

        Returns:
            The API response

        Raises:
            AdonisAuthenticationDeniedError: If authentication fails
            AdonisError: If the API returns an error
        """
        # Clone the payload for cache lookup (before token is added)
        cache_payload = payload.copy()

        # Check cache if enabled and applicable for this request
        if self.cache_enabled and use_cache:
            cached_response = self.cache.get(endpoint, cache_payload)
            if cached_response:
                logger.info(f"Using cached response for {endpoint}")
                return cached_response

        base_url = (
            self.crew_portal_base_url
            if api == CREW_PORTAL
            else self.integration_base_url
        )
        url = urljoin(base_url, endpoint)

        req = Request(url)
        req.add_header("User-Agent", USER_AGENT)
        req.add_header("Content-Type", "application/json")
        req.add_header("Accept", "application/json")

        if add_token:
            payload["Authentication_Token"] = self.getToken(api=api)

        if wrap_request:
            payload = {"request": payload}

        req.data = json.dumps(payload).encode()

        if print_payload:
            logger.debug(f"Request payload: {req.data.decode()}")

        res = json.loads(urlopen(req).read())

        if extract_response:
            res = res[endpoint + "Result"]

        if not res["Authentication_Approved"]:
            raise AdonisAuthenticationDeniedError(res["Authentication_ReasonDenied"])

        if res["ErrorText"]:
            raise AdonisError(res["ErrorText"])

        # Store in cache if enabled and applicable
        if self.cache_enabled and use_cache:
            self.cache.set(endpoint, cache_payload, res)

        return res

    def requestIntegration(
        self,
        endpoint: str,
        payload: Dict[str, Any] = {},
        add_token: bool = True,
        wrap_request: bool = True,
        extract_response: bool = True,
        print_payload: bool = False,
        use_cache: bool = False,
    ) -> Dict[str, Any]:
        """Make a request to the Integration API."""
        return self.request(
            endpoint,
            payload,
            add_token,
            wrap_request,
            extract_response,
            INTEGRATION,
            print_payload,
            use_cache,
        )

    def requestCrewPortal(
        self,
        endpoint: str,
        payload: Dict[str, Any],
        add_token: bool = True,
        wrap_request: bool = True,
        extract_response: bool = True,
        use_cache: bool = False,
    ) -> Dict[str, Any]:
        """Make a request to the Crew Portal API."""
        return self.request(
            endpoint,
            payload,
            add_token,
            wrap_request,
            extract_response,
            CREW_PORTAL,
            False,
            use_cache,
        )

    def getToken(self, lifetime: int = 60, api: int = CREW_PORTAL):
        data = {
            "credentials": {
                "Login": self.login,
                "Password": self.password,
                "LifeTime": lifetime,
            }
        }

        method = (
            self.requestCrewPortal if api == CREW_PORTAL else self.requestIntegration
        )
        result = method("GNL_API_AUTHENTICATION", data, False, False)

        return result["Authentication_Token"]

    def getCrewPersonalData(self, pin):
        data = {"Pin": pin}

        result = self.requestCrewPortal("DG_GeneralDetailsRead", data)

        return result

    def getCrewProfilePicture(self, pin):
        data = {"Pin": pin}

        try:
            result = self.requestCrewPortal("DG_ProfilePictureRead", data)
        except AdonisError:
            return None

        return result["FileBase64"]

    def getCrew(self, pin, personal_data=True, profile_picture=True):
        crew = Crew(pin)

        if personal_data:
            crew._personal_data = self.getCrewPersonalData(pin)

        if profile_picture:
            crew._profile_picture = self.getCrewProfilePicture(pin)

        return crew

    def getCrewListView(
        self,
        view: str,
        filters: str = "",
        pagination: str = "",
        paginate_by: str = "",
        page: str = "",
    ) -> Dict[str, Any]:
        """
        Get a crew list view.

        Args:
            view: View ID
            filters: Filter string
            pagination: Pagination type
            paginate_by: Number of rows per page
            page: Page number

        Returns:
            The view data
        """
        data = {
            "View": view,
            "Filter": filters,
            "Pagination": pagination or "None",
            "RowsByPage": paginate_by,
            "Page": page,
        }

        try:
            result = self.requestIntegration(
                "GNL_APMCrewListViews", data, use_cache=True
            )
        except AdonisError:
            result = None

        return result

    def getRequirements(self, simple=True):
        requirements = []
        page = self.getCrewListView(REQUIREMENTS_VIEW)

        for record in json.loads(page["Result"]):
            if simple:
                requirements.append((record["PIN"], record["Document code"]))
            else:
                requirements.append(record)

        return requirements

    def getCodes(self):
        try:
            return self.requestIntegration("DG_GetCodes")
        except AdonisError:
            return None

    def writeCompetence(
        self,
        pin,
        competence_code,
        certificate_number,
        from_timestamp=None,
        to_timestamp=None,
        issued_by="pyadonis",
        attachment=None,
        **kwargs,
    ):
        from_dt = (
            datetime.datetime.fromtimestamp(from_timestamp)
            if from_timestamp
            else datetime.datetime.now()
        )
        from_dt = from_dt.replace(hour=0, minute=0, second=0, microsecond=0)
        from_dt_string = from_dt.isoformat()

        if to_timestamp:
            to_dt = datetime.datetime.fromtimestamp(to_timestamp)
        else:
            codes = self.getCodes()
            expiry_days = 0

            for competence in codes["Competence"]:
                if competence["Code"] == competence_code:
                    expiry_days = int(competence["DefaultExpiryDays"])

            if expiry_days == 0:
                to_dt_string = ""
            else:
                if not expiry_days % 365:
                    to_dt = from_dt + relativedelta(years=expiry_days / 365)
                elif not expiry_days % 30:
                    to_dt = from_dt + relativedelta(months=expiry_days / 30)
                else:
                    to_dt = from_dt + relativedelta(days=expiry_days)

                to_dt_string = to_dt.isoformat()

        payload = {
            "Pin": pin,
            "Code": competence_code,
            "Number": certificate_number,
            "DateFrom": from_dt_string,
            "ExpiryDate": to_dt_string,
            "IssuedBy": issued_by,
            "IssuedWhere": issued_by,
            "Comment": "",
            "ScanValidity": "1",
        }

        payload.update(kwargs)

        response = self.requestIntegration("DG_CompetenceAdd", payload)

        if not attachment:
            return response

        attachment_filename = "certificate.pdf"

        if isinstance(attachment, bytes):
            attachment = attachment.decode()

        attachment_base64 = "data:application/pdf;base64," + attachment

        attachment_payload = {
            "Pin": pin,
            "Sequenceno": response["Sequenceno"],
            "docName": attachment_filename,
            "docType": "CERT",
            "Description": competence_code,
            "FileBase64": attachment_base64,
        }

        return self.requestIntegration(
            "DG_CompetenceSaveEnclosedDoc", attachment_payload
        )

    def getCompetences(self, pin=None):
        if not pin:
            certs = self.getCrewListView(CERTIFICATES_VIEW)  # ["Result"]
        else:
            certs = self.getCrewListView(CERTIFICATES_VIEW, filters="Pin=" + pin)
        outcerts = []
        for cert in json.loads(certs["Result"]):
            cert["Document code"] = cert["Document Code"]
            cert["Issued date"] = cert["Create Date"]
            outcerts.append(cert)
        # return [cert for cert in json.loads(certs) if ((not pin) or (pin == cert["PIN"]))]
        return outcerts

    def clearCache(
        self, endpoint: Optional[str] = None, params: Optional[Dict[str, Any]] = None
    ) -> None:
        """
        Clear cache entries.

        Args:
            endpoint: If provided, clear only cache for this endpoint
            params: If provided with endpoint, clear only cache for this specific request
        """
        if self.cache_enabled and self.cache:
            self.cache.clear(endpoint, params)
