"""
Cache management for PyAdonis API responses.
"""

import os
import json
import time
import hashlib
import logging
from pathlib import Path
from typing import Optional, Dict, Any

logger = logging.getLogger(__name__)

class ResponseCache:
    """Cache handler for API responses."""
    
    def __init__(
        self, 
        cache_dir: Optional[str] = None,
        expiry_seconds: int = 3600  # Default: 1 hour
    ):
        """
        Initialize the cache handler.
        
        Args:
            cache_dir: Directory to store cache files. Defaults to ~/.pyadonis/cache
            expiry_seconds: How long cached data is considered valid (in seconds)
        """
        if cache_dir is None:
            self.cache_dir = Path.home() / '.pyadonis' / 'cache'
        else:
            self.cache_dir = Path(cache_dir)
            
        self.expiry_seconds = expiry_seconds
        
        # Create cache directory if it doesn't exist
        os.makedirs(self.cache_dir, exist_ok=True)
        
    def _generate_cache_key(self, endpoint: str, params: Dict[str, Any]) -> str:
        """
        Generate a unique cache key based on the endpoint and parameters.
        
        Args:
            endpoint: The API endpoint
            params: Dictionary of parameters
            
        Returns:
            A unique hash string to use as the cache key
        """
        # Create a string representation of the request
        request_repr = f"{endpoint}:{json.dumps(params, sort_keys=True)}"
        
        # Create a hash of the request
        return hashlib.md5(request_repr.encode('utf-8')).hexdigest()
    
    def _get_cache_path(self, cache_key: str) -> Path:
        """Get the file path for a cache key."""
        return self.cache_dir / f"{cache_key}.json"
    
    def get(self, endpoint: str, params: Dict[str, Any]) -> Optional[Dict[str, Any]]:
        """
        Retrieve data from cache if it exists and is not expired.
        
        Args:
            endpoint: API endpoint
            params: Request parameters
            
        Returns:
            The cached data if valid, None otherwise
        """
        cache_key = self._generate_cache_key(endpoint, params)
        cache_path = self._get_cache_path(cache_key)
        
        if not cache_path.exists():
            logger.debug(f"Cache miss: {endpoint}")
            return None
        
        # Check if cache is expired
        file_mtime = cache_path.stat().st_mtime
        if time.time() - file_mtime > self.expiry_seconds:
            logger.debug(f"Cache expired: {endpoint}")
            return None
        
        # Load and return cached data
        try:
            with open(cache_path, 'r') as f:
                logger.debug(f"Cache hit: {endpoint}")
                return json.load(f)
        except Exception as e:
            logger.warning(f"Error reading cache file: {e}")
            return None
    
    def set(self, endpoint: str, params: Dict[str, Any], data: Dict[str, Any]) -> None:
        """
        Store data in the cache.
        
        Args:
            endpoint: API endpoint
            params: Request parameters
            data: Data to cache
        """
        cache_key = self._generate_cache_key(endpoint, params)
        cache_path = self._get_cache_path(cache_key)
        
        try:
            with open(cache_path, 'w') as f:
                json.dump(data, f)
            logger.debug(f"Cached response for: {endpoint}")
        except Exception as e:
            logger.warning(f"Error writing to cache: {e}")
    
    def clear(self, endpoint: Optional[str] = None, params: Optional[Dict[str, Any]] = None) -> None:
        """
        Clear specific cache entries or the entire cache.
        
        Args:
            endpoint: If provided, clear only cache for this endpoint
            params: If provided with endpoint, clear only cache for this specific request
        """
        if endpoint and params:
            # Clear specific cache entry
            cache_key = self._generate_cache_key(endpoint, params)
            cache_path = self._get_cache_path(cache_key)
            if cache_path.exists():
                os.remove(cache_path)
                logger.debug(f"Cleared cache for: {endpoint}")
        elif endpoint:
            # TODO: Make this possible by storing the endpoint in the file
            pass
        else:
            # Clear all cache
            for cache_file in self.cache_dir.glob("*.json"):
                os.remove(cache_file)
            logger.debug("Cleared all cache")