"""
Command-line tool to manage the PyAdonis cache.
"""

from argparse import ArgumentParser
from pathlib import Path

from ..config import Config
from ..api import Adonis


def main():
    parser = ArgumentParser(description="Manage PyAdonis cache")
    subparsers = parser.add_subparsers(dest="command", help="Command")

    # Clear cache command
    clear_parser = subparsers.add_parser("clear", help="Clear cache")
    clear_parser.add_argument(
        "--endpoint", help="Clear cache only for specific endpoint"
    )

    # Info command
    info_parser = subparsers.add_parser("info", help="Show cache information")

    args = parser.parse_args()

    if not args.command:
        parser.print_help()
        return

    config = Config()

    if not config.cache_enabled:
        print("Caching is disabled in settings.ini")
        return

    # Create API client (for cache configuration)
    api = Adonis.fromConfig(config)

    if args.command == "clear":
        if args.endpoint:
            api.clearCache(endpoint=args.endpoint)
            print(f"Cleared cache for endpoint: {args.endpoint}")
        else:
            api.clearCache()
            print("Cleared all cache")

    elif args.command == "info":
        cache_dir = Path(api.cache.cache_dir)
        if not cache_dir.exists():
            print(f"Cache directory does not exist: {cache_dir}")
            return

        cache_files = list(cache_dir.glob("*.json"))
        total_size = sum(f.stat().st_size for f in cache_files)

        print(f"Cache location: {cache_dir}")
        print(f"Cache entries: {len(cache_files)}")
        print(f"Total size: {total_size / 1024:.2f} KB")
        print(f"Expiry time: {api.cache.expiry_seconds} seconds")


if __name__ == "__main__":
    main()
