"""
Command-line tool to fetch requirements.
"""

from argparse import ArgumentParser
import sys
import json

from ..config import Config
from ..api import Adonis


def main():
    parser = ArgumentParser(description="Fetch requirements")
    parser.add_argument(
        "--full", action="store_true", help="Show full requirement details"
    )
    parser.add_argument(
        "--count", action="store_true", help="Show only count of requirements"
    )
    parser.add_argument(
        "-f",
        "--format",
        choices=["json", "table"],
        default="json",
        help="Output format (default: json)",
    )
    args = parser.parse_args()

    config = Config()
    api = Adonis.fromConfig(config)

    try:
        data = api.getRequirements(not args.full)

        if args.count:
            print(f"Total requirements: {len(data)}")
            return 0

        if args.format == "json":
            print(json.dumps(data, indent=2))
        else:  # table format
            if not data:
                print("No requirements found.")
                return 0

            if args.full:
                # Full data is a list of dictionaries
                headers = list(data[0].keys())

                # Calculate column widths
                col_widths = {
                    header: max(
                        len(header),
                        max(len(str(item.get(header, ""))) for item in data),
                    )
                    for header in headers
                }

                # Print header
                header_row = " | ".join(
                    f"{header:{col_widths[header]}}" for header in headers
                )
                print(header_row)
                print("-" * len(header_row))

                # Print data rows
                for item in data:
                    row = " | ".join(
                        f"{str(item.get(header, '')):{col_widths[header]}}"
                        for header in headers
                    )
                    print(row)
            else:
                # Simple data is a list of (pin, code) tuples
                print("PIN | Document Code")
                print("-" * 30)
                for pin, code in data:
                    print(f"{pin} | {code}")

        return 0
    except Exception as e:
        print(f"Error: {e}")
        return 1


if __name__ == "__main__":
    sys.exit(main())
