"""
Command-line tool to create or update competence certificates.
"""

from argparse import ArgumentParser
import sys
import base64
from datetime import datetime

from ..config import Config
from ..api import Adonis


def main():
    parser = ArgumentParser(description="Create or update competence certificates")
    parser.add_argument("pin", help="PIN of the crew member")
    parser.add_argument("competence", help="Competence code")
    parser.add_argument("-n", "--number", default="", help="Certificate number")
    parser.add_argument("-i", "--instance", default="MS4scA", help="Issuing instance")
    parser.add_argument(
        "-f", "--from-timestamp", type=int, help="From timestamp (Unix time)"
    )
    parser.add_argument(
        "-t", "--to-timestamp", type=int, help="To timestamp (Unix time)"
    )
    parser.add_argument("-F", "--file", help="Path to certificate file (PDF)")
    parser.add_argument("--from-date", help="From date (YYYY-MM-DD)")
    parser.add_argument("--to-date", help="To date (YYYY-MM-DD)")
    args = parser.parse_args()

    config = Config()
    api = Adonis.fromConfig(config)

    attachment = None

    # Process file if provided
    if args.file:
        try:
            with open(args.file, "rb") as certfile:
                attachment = base64.encodebytes(certfile.read())
        except Exception as e:
            print(f"Error reading file: {e}")
            return 1

    # Process from and to timestamps
    from_timestamp = args.from_timestamp
    to_timestamp = args.to_timestamp

    # Override timestamps with date strings if provided
    if args.from_date:
        try:
            dt = datetime.strptime(args.from_date, "%Y-%m-%d")
            from_timestamp = int(dt.timestamp())
        except ValueError:
            print("Invalid from-date format. Use YYYY-MM-DD")
            return 1

    if args.to_date:
        try:
            dt = datetime.strptime(args.to_date, "%Y-%m-%d")
            to_timestamp = int(dt.timestamp())
        except ValueError:
            print("Invalid to-date format. Use YYYY-MM-DD")
            return 1

    try:
        result = api.writeCompetence(
            args.pin,
            args.competence,
            args.number,
            issued_by=args.instance,
            from_timestamp=from_timestamp,
            to_timestamp=to_timestamp,
            attachment=attachment,
        )

        print(f"Certificate created successfully for PIN {args.pin}")
        print(f"Competence: {args.competence}")
        if "Sequenceno" in result:
            print(f"Sequence number: {result['Sequenceno']}")

        return 0
    except Exception as e:
        print(f"Error: {e}")
        return 1


if __name__ == "__main__":
    sys.exit(main())
