"""
Configuration handler for the PyAdonis client.
"""

from configparser import ConfigParser
from typing import Optional


class Config:
    """
    Configuration handler that reads settings from an INI file.
    """

    def __init__(self, path: Optional[str] = "/etc/pyadonis/settings.ini"):
        """
        Initialize with path to configuration file.

        Args:
            path: Path to the INI configuration file
        """
        self.config = ConfigParser()
        self.config.read(path)

    @property
    def crew_portal_base_url(self) -> str:
        """URL for the Crew Portal API"""
        return self.config["ADONIS"]["CrewPortalBaseURL"]

    @property
    def integration_base_url(self) -> str:
        """URL for the Integration API"""
        return self.config.get(
            "ADONIS", "IntegrationBaseURL", fallback=self.crew_portal_base_url
        )

    @property
    def login(self) -> str:
        """API login username"""
        return self.config["ADONIS"]["Login"]

    @property
    def password(self) -> str:
        """API login password"""
        return self.config["ADONIS"]["Password"]

    @property
    def cache_enabled(self) -> bool:
        """Whether caching is enabled"""
        return self.config.getboolean("CACHE", "Enabled", fallback=True)

    @property
    def cache_dir(self) -> Optional[str]:
        """Directory to store cache files"""
        return self.config.get("CACHE", "Directory", fallback=None)

    @property
    def cache_expiry(self) -> int:
        """Cache expiry time in seconds"""
        return self.config.getint("CACHE", "ExpirySeconds", fallback=3600)
